@@ -267,8 +267,7 @@ class Configuration(object):
 
         changed = False
         modtime = os.path.getmtime(self.filename)
-        if force or modtime > self._lastmtime:
-            self._sections = {}
+        if force or modtime != self._lastmtime:
             self.parser._sections = {}
             if not self.parser.read(self.filename):
                 raise TracError(_("Error reading '%(file)s', make sure it is "
@@ -291,14 +290,13 @@ class Configuration(object):
                 changed |= parent.parse_if_needed(force=force)
 
         if changed:
-            self._cache = {}
+            self._sections = {}
         return changed
 
     def touch(self):
         if self.filename and os.path.isfile(self.filename) \
                 and os.access(self.filename, os.W_OK):
             wait_for_file_mtime_change(self.filename)
-            os.utime(self.filename, None)
 
     def set_defaults(self, compmgr=None):
         """Retrieve all default values and store them explicitly in the
